#include "api.h"
#include "cos_api.h"
#include "cos_sys_config.h"
#include "cos_config.h"
#include "util/auth_tool.h"
#include "Poco/TaskManager.h"

#include <stdlib.h>
#include <sys/stat.h>
#include <iostream>
#include <map>
#include <string>
#include <thread>
#include <vector>

using namespace qcloud_cos;
CosConfig g_config;

void result2ufileError(ufile_error & retError, qcloud_cos::CosResult tRet)
{
	retError.errorCode = tRet.GetErrorCode();
	retError.errorMessage = tRet.GetErrorMsg();
	retError.resourceAddr = tRet.GetResourceAddr();
	retError.cosRequestId = tRet.GetXCosRequestId();
	retError.cosTraceId = tRet.GetXCosTraceId();
	retError.httpStatus = tRet.GetHttpStatus();
	retError.bIsSuccess = tRet.IsSucc();
}

ufile_error ufile_load_config_from_json(const char * json_buf, ufile_config * cfg)
{
	ufile_error retError;
	if (nullptr == cfg)
		return retError;
	Poco::JSON::Parser parser;
	Poco::Dynamic::Var result;
	try {
		result = parser.parse(json_buf);
	}
	catch (Poco::JSON::JSONException& jsonErr) {
		retError.errorMessage = jsonErr.message();
		retError.bIsSuccess = false;
		return retError;
	}
	if (result.type() != typeid(Poco::JSON::Object::Ptr)) {
		retError.errorMessage = "failed to parse config file";
		retError.bIsSuccess = false;
		return retError;
	}
	Poco::JSON::Object::Ptr object = result.extract<Poco::JSON::Object::Ptr>();

	uint64_t appid = 0;
	std::string strPublicKey = "";
	std::string strPrivateKey = "";
	std::string strRegion = "";
	CosConfig::JsonObjectGetIntegerValue(object, "AppID", &appid);
	CosConfig::JsonObjectGetStringValue(object, "AccessKey", &strPublicKey);
	CosConfig::JsonObjectGetStringValue(object, "SecretId", &strPublicKey);
	CosConfig::JsonObjectGetStringValue(object, "SecretKey", &strPrivateKey);
	CosConfig::JsonObjectGetStringValue(object, "Region", &strRegion);
	cfg->appid = appid;
	cfg->public_key = strPublicKey.c_str();
	cfg->private_key = strPrivateKey.c_str();
	cfg->region = strRegion.c_str();
	retError.bIsSuccess = true;
	return retError;
}

void ufile_free_config(ufile_config cfg)
{
	//˺޵ãҲʵ
}

ufile_error ufile_sdk_initialize(const ufile_config cfg, int open_verbose)
{
	UNUSED_PARAM(open_verbose);
	ufile_error retError;
	CosConfig config(cfg.appid, cfg.public_key, cfg.private_key, cfg.region);
	g_config = config;
	retError.bIsSuccess = true;
	return retError;
}

void ufile_sdk_cleanup()
{
}

ufile_error ufile_head(const char * bucket_name, const char * key, ufile_file_info * info)
{
	//˺޵ãҲʵ
	return ufile_error();
}

void ufile_free_file_info(ufile_file_info info)
{
	//˺޵ãҲʵ
}

ufile_error ufile_put_buf(const char * bucket_name, const char * key, const char * mime_type, char * buffer, size_t buf_len)
{
	ufile_error retError;

	return ufile_error();
}

ufile_error ufile_put_file(const char * bucket_name, const char * key, const char * mime_type, FILE * file)
{
	ufile_error retError;

	return ufile_error();
}

ufile_error ufile_put_file_by_filepath(const char * bucket_name, const char * key, const char * mime_type, const char * file_path)
{
	UNUSED_PARAM(mime_type);
	ufile_error retError;
	qcloud_cos::PutObjectByFileReq req(bucket_name, key, file_path);
	qcloud_cos::PutObjectByFileResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.PutObject(req, &resp);
	result2ufileError(retError, result);
	return retError;
}

void set_upload_company(int company)
{
}

ufile_error ufile_delete(const char * bucket_name, const char * key)
{
	ufile_error retError;
	qcloud_cos::DeleteObjectReq req(bucket_name, key);
	qcloud_cos::DeleteObjectResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.DeleteObject(req, &resp);
	result2ufileError(retError, result);
	return ufile_error();
}

ufile_error ufile_multiple_upload_init(ufile_mutipart_state * self, const char * bucket_name, const char * key, const char * mime_type)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_part(ufile_mutipart_state * self, char * buffer, size_t buf_len, int part_number)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_finish(ufile_mutipart_state * self)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_multiple_upload_abort(ufile_mutipart_state * self)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_download(const char * bucket_name, const char * key, FILE * file, size_t * return_size, const char * path)
{
	ufile_error retError;
	if (nullptr == file)
	{
		retError.errorMessage = "file is nullptr";
		return retError;
	}
	std::ostringstream os;
	qcloud_cos::GetObjectByStreamReq req(bucket_name, key, os);
	qcloud_cos::GetObjectByStreamResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.GetObject(req, &resp);
	if (result.IsSucc())
	{
		const char * tBuf = os.str().c_str();
		fwrite(tBuf, sizeof(tBuf), strlen(tBuf), file);
	}
	result2ufileError(retError, result);
	return retError;
}

ufile_error ufile_download_piece(const char * bucket_name, const char * key, size_t start_position, char * buf, size_t buf_len, size_t * return_size)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_bucket_create(const char * bucket_name, const char * region, const char * bucket_type)
{
	//˺޵ãҲʵ
	return ufile_error();
}

ufile_error ufile_bucket_delete(const char * bucket_name)
{
	ufile_error retError;
	qcloud_cos::DeleteBucketReq req(bucket_name);
	qcloud_cos::DeleteBucketResp resp;
	CosAPI cosApi(g_config);
	CosResult result = cosApi.DeleteBucket(req, &resp);
	result2ufileError(retError, result);
	return ufile_error();
}

